package sun.tools.jar;

import java.util.zip.CRC32;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.List;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.nio.file.CopyOption;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.util.Enumeration;
import java.util.zip.ZipFile;
import java.util.jar.Attributes;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipInputStream;
import sun.misc.JarIndex;
import java.io.InputStream;
import java.util.Locale;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.io.OutputStream;
import java.io.FileNotFoundException;
import java.util.jar.JarOutputStream;
import java.util.jar.JarFile;
import java.util.jar.Pack200;
import java.util.jar.Manifest;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileDescriptor;
import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.HashMap;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.HashSet;
import java.util.ResourceBundle;
import java.util.Set;
import java.io.File;
import java.util.Map;
import java.io.PrintStream;

public class Main
{
    String program;
    PrintStream out;
    PrintStream err;
    String fname;
    String mname;
    String ename;
    String zname;
    String[] files;
    String rootjar;
    Map<String, File> entryMap;
    Set<File> entries;
    Set<String> paths;
    boolean cflag;
    boolean uflag;
    boolean xflag;
    boolean tflag;
    boolean vflag;
    boolean flag0;
    boolean Mflag;
    boolean iflag;
    boolean nflag;
    boolean pflag;
    static final String MANIFEST_DIR = "META-INF/";
    static final String VERSION = "1.0";
    private static ResourceBundle rsrc;
    private static final boolean useExtractionTime;
    private boolean ok;
    private byte[] copyBuf;
    private HashSet<String> jarPaths;
    
    private String getMsg(String key) {
        try {
            return Main.rsrc.getString(key);
        }
        catch (MissingResourceException ex) {
            throw new Error("Error in message file");
        }
    }
    
    private String formatMsg(String s, String s2) {
        return MessageFormat.format(this.getMsg(s), (Object[])new String[] { s2 });
    }
    
    private String formatMsg2(String s, String s2, String s3) {
        return MessageFormat.format(this.getMsg(s), (Object[])new String[] { s2, s3 });
    }
    
    public Main(PrintStream out, PrintStream err, String program) {
        this.zname = "";
        this.rootjar = null;
        this.entryMap = new HashMap();
        this.entries = new LinkedHashSet();
        this.paths = new HashSet();
        this.copyBuf = new byte[8192];
        this.jarPaths = new HashSet();
        this.out = out;
        this.err = err;
        this.program = program;
    }
    
    private static File createTempFileInSameDirectoryAs(File file) throws IOException {
        File parentFile = file.getParentFile();
        if (parentFile == null) {
            parentFile = new File(".");
        }
        return File.createTempFile("jartmp", null, parentFile);
    }
    
    public synchronized boolean run(String[] array) {
        this.ok = true;
        if (!this.parseArgs(array)) {
            return false;
        }
        try {
            if ((this.cflag || this.uflag) && this.fname != null) {
                this.zname = this.fname.replace(File.separatorChar, '/');
                if (this.zname.startsWith("./")) {
                    this.zname = this.zname.substring(2);
                }
            }
            if (this.cflag) {
                Manifest manifest = null;
                FileInputStream in3 = null;
                if (!this.Mflag) {
                    if (this.mname != null) {
                        in3 = new FileInputStream(this.mname);
                        manifest = new Manifest(new BufferedInputStream(in3));
                    }
                    else {
                        manifest = new Manifest();
                    }
                    this.addVersion(manifest);
                    this.addCreatedBy(manifest);
                    if (this.isAmbiguousMainClass(manifest)) {
                        if (in3 != null) {
                            in3.close();
                        }
                        return false;
                    }
                    else if (this.ename != null) {
                        this.addMainClass(manifest, this.ename);
                    }
                }
                this.expand(null, this.files, false);
                Object o2;
                if (this.fname != null) {
                    o2 = new FileOutputStream(this.fname);
                }
                else {
                    o2 = new FileOutputStream(FileDescriptor.out);
                    if (this.vflag) {
                        this.vflag = false;
                    }
                }
                File temporaryFile = null;
                String s = (this.fname == null) ? "tmpjar" : this.fname.substring(this.fname.indexOf((int)File.separatorChar) + 1);
                if (this.nflag) {
                    temporaryFile = this.createTemporaryFile(s, ".jar");
                    o2 = new FileOutputStream(temporaryFile);
                }
                this.create(new BufferedOutputStream(o2, 4096), manifest);
                if (in3 != null) {
                    in3.close();
                }
                o2.close();
                if (this.nflag) {
                    Object o3 = null;
                    File temporaryFile2 = null;
                    Object o4 = null;
                    try {
                        Pack200.Packer packer = Pack200.newPacker();
                        packer.properties().put("pack.effort", "1");
                        o3 = new JarFile(temporaryFile.getCanonicalPath());
                        temporaryFile2 = this.createTemporaryFile(s, ".pack");
                        o2 = new FileOutputStream(temporaryFile2);
                        packer.pack(o3, o2);
                        o4 = new JarOutputStream(o2);
                        Pack200.newUnpacker().unpack(temporaryFile2, o4);
                    }
                    catch (IOException ex) {
                        this.fatalError(ex);
                    }
                    finally {
                        if (o3 != null) {
                            o3.close();
                        }
                        if (o2 != null) {
                            o2.close();
                        }
                        if (o4 != null) {
                            o4.close();
                        }
                        if (temporaryFile != null && temporaryFile.exists()) {
                            temporaryFile.delete();
                        }
                        if (temporaryFile2 != null && temporaryFile2.exists()) {
                            temporaryFile2.delete();
                        }
                    }
                }
            }
            else if (this.uflag) {
                File file = null;
                File tempFileInSameDirectoryAs = null;
                FileInputStream fileInputStream;
                FileOutputStream out;
                if (this.fname != null) {
                    file = new File(this.fname);
                    tempFileInSameDirectoryAs = createTempFileInSameDirectoryAs(file);
                    fileInputStream = new FileInputStream(file);
                    out = new FileOutputStream(tempFileInSameDirectoryAs);
                }
                else {
                    fileInputStream = new FileInputStream(FileDescriptor.in);
                    out = new FileOutputStream(FileDescriptor.out);
                    this.vflag = false;
                }
                Object o = (!this.Mflag && this.mname != null) ? new FileInputStream(this.mname) : null;
                this.expand(null, this.files, true);
                int update = this.update(fileInputStream, new BufferedOutputStream(out), o, null);
                if (this.ok) {
                    this.ok = update;
                }
                fileInputStream.close();
                out.close();
                if (o != null) {
                    o.close();
                }
                if (this.ok && this.fname != null) {
                    file.delete();
                    if (!tempFileInSameDirectoryAs.renameTo(file)) {
                        tempFileInSameDirectoryAs.delete();
                        throw new IOException(this.getMsg("error.write.file"));
                    }
                    tempFileInSameDirectoryAs.delete();
                }
            }
            else if (this.tflag) {
                this.replaceFSC(this.files);
                if (this.fname != null) {
                    this.list(this.fname, this.files);
                }
                else {
                    FileInputStream in2 = new FileInputStream(FileDescriptor.in);
                    try {
                        this.list(new BufferedInputStream(in2), this.files);
                    }
                    finally {
                        in2.close();
                    }
                }
            }
            else if (this.xflag) {
                this.replaceFSC(this.files);
                if (this.fname != null && this.files != null) {
                    this.extract(this.fname, this.files);
                }
                else {
                    FileInputStream in = (this.fname == null) ? new FileInputStream(FileDescriptor.in) : new FileInputStream(this.fname);
                    try {
                        this.extract(new BufferedInputStream(in), this.files);
                    }
                    finally {
                        in.close();
                    }
                }
            }
            else if (this.iflag) {
                this.genIndex(this.rootjar, this.files);
            }
        }
        catch (IOException ex2) {
            this.fatalError(ex2);
            this.ok = false;
        }
        catch (Error error) {
            error.printStackTrace();
            this.ok = false;
        }
        catch (Throwable t) {
            t.printStackTrace();
            this.ok = false;
        }
        this.out.flush();
        this.err.flush();
        return this.ok;
    }
    
    boolean parseArgs(String[] parse) {
        try {
            parse = CommandLine.parse(parse);
        }
        catch (FileNotFoundException ex) {
            this.fatalError(this.formatMsg("error.cant.open", ex.getMessage()));
            return false;
        }
        catch (IOException ex2) {
            this.fatalError(ex2);
            return false;
        }
        int i = 1;
        try {
            String substring = parse[0];
            if (substring.startsWith("-")) {
                substring = substring.substring(1);
            }
            for (int j = 0; j < substring.length(); ++j) {
                switch (substring.charAt(j)) {
                    case 'c': {
                        if (this.xflag || this.tflag || this.uflag || this.iflag) {
                            this.usageError();
                            return false;
                        }
                        this.cflag = true;
                        break;
                    }
                    case 'u': {
                        if (this.cflag || this.xflag || this.tflag || this.iflag) {
                            this.usageError();
                            return false;
                        }
                        this.uflag = true;
                        break;
                    }
                    case 'x': {
                        if (this.cflag || this.uflag || this.tflag || this.iflag) {
                            this.usageError();
                            return false;
                        }
                        this.xflag = true;
                        break;
                    }
                    case 't': {
                        if (this.cflag || this.uflag || this.xflag || this.iflag) {
                            this.usageError();
                            return false;
                        }
                        this.tflag = true;
                        break;
                    }
                    case 'M': {
                        this.Mflag = true;
                        break;
                    }
                    case 'v': {
                        this.vflag = true;
                        break;
                    }
                    case 'f': {
                        this.fname = parse[i++];
                        break;
                    }
                    case 'm': {
                        this.mname = parse[i++];
                        break;
                    }
                    case '0': {
                        this.flag0 = true;
                        break;
                    }
                    case 'i': {
                        if (this.cflag || this.uflag || this.xflag || this.tflag) {
                            this.usageError();
                            return false;
                        }
                        this.rootjar = parse[i++];
                        this.iflag = true;
                        break;
                    }
                    case 'n': {
                        this.nflag = true;
                        break;
                    }
                    case 'e': {
                        this.ename = parse[i++];
                        break;
                    }
                    case 'P': {
                        this.pflag = true;
                        break;
                    }
                    default: {
                        this.error(this.formatMsg("error.illegal.option", String.valueOf(substring.charAt(j))));
                        this.usageError();
                        return false;
                    }
                }
            }
        }
        catch (ArrayIndexOutOfBoundsException ex3) {
            this.usageError();
            return false;
        }
        if (!this.cflag && !this.tflag && !this.xflag && !this.uflag && !this.iflag) {
            this.error(this.getMsg("error.bad.option"));
            this.usageError();
            return false;
        }
        int n = parse.length - i;
        if (n > 0) {
            int n2 = 0;
            String[] array = new String[n];
            try {
                for (; i < parse.length; ++i) {
                    if (parse[i].equals("-C")) {
                        String str = parse[++i];
                        String str2;
                        for (str2 = (str.endsWith(File.separator) ? str : (str + File.separator)).replace(File.separatorChar, '/'); str2.indexOf("//") > -1; str2 = str2.replace("//", "/")) {}
                        this.paths.add(str2.replace(File.separatorChar, '/'));
                        array[n2++] = str2 + parse[++i];
                    }
                    else {
                        array[n2++] = parse[i];
                    }
                }
            }
            catch (ArrayIndexOutOfBoundsException ex4) {
                this.usageError();
                return false;
            }
            System.arraycopy(array, 0, this.files = new String[n2], 0, n2);
        }
        else {
            if (this.cflag && this.mname == null) {
                this.error(this.getMsg("error.bad.cflag"));
                this.usageError();
                return false;
            }
            if (this.uflag) {
                if (this.mname != null || this.ename != null) {
                    return true;
                }
                this.error(this.getMsg("error.bad.uflag"));
                this.usageError();
                return false;
            }
        }
        return true;
    }
    
    void expand(File parent, String[] array, boolean b) {
        if (array == null) {
            return;
        }
        for (int i = 0; i < array.length; ++i) {
            Object obj;
            if (parent == null) {
                obj = new File(array[i]);
            }
            else {
                obj = new File(parent, array[i]);
            }
            if (obj.isFile()) {
                if (this.entries.add(obj) && b) {
                    this.entryMap.put(this.entryName(obj.getPath()), obj);
                }
            }
            else if (obj.isDirectory()) {
                if (this.entries.add(obj)) {
                    if (b) {
                        String path = obj.getPath();
                        this.entryMap.put(this.entryName(path.endsWith(File.separator) ? path : (path + File.separator)), obj);
                    }
                    this.expand(obj, obj.list(), b);
                }
            }
            else {
                this.error(this.formatMsg("error.nosuch.fileordir", String.valueOf(obj)));
                this.ok = false;
            }
        }
        return;
    }
    
    void create(OutputStream out, Manifest manifest) throws IOException {
        JarOutputStream out2 = new JarOutputStream(out);
        if (this.flag0) {
            out2.setMethod(0);
        }
        if (manifest != null) {
            if (this.vflag) {
                this.output(this.getMsg("out.added.manifest"));
            }
            ZipEntry e = new ZipEntry("META-INF/");
            e.setTime(System.currentTimeMillis());
            e.setSize(0L);
            e.setCrc(0L);
            out2.putNextEntry(e);
            ZipEntry e2 = new ZipEntry("META-INF/MANIFEST.MF");
            e2.setTime(System.currentTimeMillis());
            if (this.flag0) {
                this.crc32Manifest(e2, manifest);
            }
            out2.putNextEntry(e2);
            manifest.write(out2);
            out2.closeEntry();
        }
        Iterator<File> iterator = this.entries.iterator();
        while (iterator.hasNext()) {
            this.addFile(out2, (File)iterator.next());
        }
        out2.close();
    }
    
    private char toUpperCaseASCII(char c) {
        return (c < 'a' || c > 'z') ? c : ((char)(c + 65 - 97));
    }
    
    private boolean equalsIgnoreCase(String s, String anObject) {
        assert !(!anObject.toUpperCase(Locale.ENGLISH).equals(anObject));
        int length;
        if ((length = s.length()) != anObject.length()) {
            return false;
        }
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i);
            int char2 = anObject.charAt(i);
            if (char1 != char2 && this.toUpperCaseASCII((char)char1) != char2) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    boolean update(InputStream in, OutputStream out, InputStream inputStream, JarIndex jarIndex) throws IOException {
        ZipInputStream is = new ZipInputStream(in);
        JarOutputStream jarOutputStream = new JarOutputStream(out);
        boolean n = false;
        boolean n2 = true;
        if (jarIndex != null) {
            this.addIndex(jarIndex, jarOutputStream);
        }
        ZipEntry nextEntry;
        while ((nextEntry = is.getNextEntry()) != null) {
            String name = nextEntry.getName();
            int equalsIgnoreCase = this.equalsIgnoreCase(name, "META-INF/MANIFEST.MF");
            if (jarIndex != null && this.equalsIgnoreCase(name, "META-INF/INDEX.LIST")) {
                continue;
            }
            if (this.Mflag && equalsIgnoreCase) {
                continue;
            }
            if (equalsIgnoreCase && (inputStream != null || this.ename != null)) {
                n = true;
                if (inputStream != null) {
                    FileInputStream is2 = new FileInputStream(this.mname);
                    int ambiguousMainClass = this.isAmbiguousMainClass(new Manifest(is2));
                    is2.close();
                    if (!(ambiguousMainClass == 0)) {
                        return false;
                    }
                }
                Manifest o = new Manifest(is);
                if (inputStream != null) {
                    o.read(inputStream);
                }
                if (!this.updateManifest(o, jarOutputStream)) {
                    return false;
                }
                continue;
            }
            else if (!this.entryMap.containsKey(name)) {
                ZipEntry e = new ZipEntry(name);
                e.setMethod(nextEntry.getMethod());
                e.setTime(nextEntry.getTime());
                e.setComment(nextEntry.getComment());
                e.setExtra(nextEntry.getExtra());
                if (nextEntry.getMethod() == 0) {
                    e.setSize(nextEntry.getSize());
                    e.setCrc(nextEntry.getCrc());
                }
                jarOutputStream.putNextEntry(e);
                this.copy(is, jarOutputStream);
            }
            else {
                File file = (File)this.entryMap.get(name);
                this.addFile(jarOutputStream, file);
                this.entryMap.remove(name);
                this.entries.remove(file);
            }
        }
        Iterator<File> iterator = this.entries.iterator();
        while (iterator.hasNext()) {
            this.addFile(jarOutputStream, (File)iterator.next());
        }
        if (n == 0) {
            if (inputStream != null) {
                Manifest o2 = new Manifest(inputStream);
                n2 = !this.isAmbiguousMainClass(o2);
                if (n2 && !this.updateManifest(o2, jarOutputStream)) {
                    n2 = false;
                }
            }
            else if (this.ename != null && !this.updateManifest(new Manifest(), jarOutputStream)) {
                n2 = false;
            }
        }
        is.close();
        jarOutputStream.close();
        return n2;
    }
    
    private void addIndex(JarIndex jarIndex, ZipOutputStream zipOutputStream) throws IOException {
        ZipEntry e = new ZipEntry("META-INF/INDEX.LIST");
        e.setTime(System.currentTimeMillis());
        if (this.flag0) {
            Main.CRC32OutputStream crc32OutputStream = new Main.CRC32OutputStream();
            jarIndex.write(crc32OutputStream);
            crc32OutputStream.updateEntry(e);
        }
        zipOutputStream.putNextEntry(e);
        jarIndex.write(zipOutputStream);
        zipOutputStream.closeEntry();
    }
    
    private boolean updateManifest(Manifest manifest, ZipOutputStream out) throws IOException {
        this.addVersion(manifest);
        this.addCreatedBy(manifest);
        if (this.ename != null) {
            this.addMainClass(manifest, this.ename);
        }
        ZipEntry e = new ZipEntry("META-INF/MANIFEST.MF");
        e.setTime(System.currentTimeMillis());
        if (this.flag0) {
            this.crc32Manifest(e, manifest);
        }
        out.putNextEntry(e);
        manifest.write(out);
        if (this.vflag) {
            this.output(this.getMsg("out.update.manifest"));
        }
        return true;
    }
    
    private static final boolean isWinDriveLetter(char c) {
        return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z');
    }
    
    private String safeName(String substring) {
        if (!this.pflag) {
            int length = substring.length();
            int i = substring.lastIndexOf("../");
            if (i == -1) {
                i = 0;
            }
            else {
                i += 3;
            }
            if (File.separatorChar == '\\') {
                while (i < length) {
                    if (i + 1 < length && substring.charAt(i + 1) == ':' && isWinDriveLetter(substring.charAt(i))) {
                        i += 2;
                    }
                    while (i < length && substring.charAt(i) == '/') {
                        ++i;
                    }
                    if (i == i) {
                        break;
                    }
                    continue;
                }
            }
            else {
                while (i < length && substring.charAt(i) == '/') {
                    ++i;
                }
            }
            if (i != 0) {
                substring = substring.substring(i);
            }
        }
        return substring;
    }
    
    private String entryName(String s) {
        s = s.replace(File.separatorChar, '/');
        String s2 = "";
        for (String prefix : this.paths) {
            if (s.startsWith(prefix) && prefix.length() > s2.length()) {
                s2 = prefix;
            }
            else {
                continue;
            }
        }
        s = s.substring(s2.length());
        s = this.safeName(s);
        if (s.startsWith("./")) {
            s = s.substring(2);
            return s;
        }
        return s;
    }
    
    private void addVersion(Manifest manifest) {
        Attributes mainAttributes = manifest.getMainAttributes();
        if (mainAttributes.getValue(Attributes.Name.MANIFEST_VERSION) == null) {
            mainAttributes.put(Attributes.Name.MANIFEST_VERSION, "1.0");
        }
    }
    
    private void addCreatedBy(Manifest manifest) {
        Attributes mainAttributes = manifest.getMainAttributes();
        if (mainAttributes.getValue(new Attributes.Name("Created-By")) == null) {
            mainAttributes.put(new Attributes.Name("Created-By"), System.getProperty("java.version") + " (" + System.getProperty("java.vendor") + ")");
        }
    }
    
    private void addMainClass(Manifest manifest, String value) {
        manifest.getMainAttributes().put(Attributes.Name.MAIN_CLASS, value);
    }
    
    private boolean isAmbiguousMainClass(Manifest manifest) {
        if (this.ename != null && manifest.getMainAttributes().get(Attributes.Name.MAIN_CLASS) != null) {
            this.error(this.getMsg("error.bad.eflag"));
            this.usageError();
            return true;
        }
        return false;
    }
    
    void addFile(ZipOutputStream zipOutputStream, File file) throws IOException {
        String path = file.getPath();
        int directory = file.isDirectory();
        if (!(directory == 0)) {
            path = (path.endsWith(File.separator) ? path : (path + File.separator));
        }
        String entryName = this.entryName(path);
        if (entryName.equals("") || entryName.equals(".") || entryName.equals(this.zname)) {
            return;
        }
        if ((entryName.equals("META-INF/") || entryName.equals("META-INF/MANIFEST.MF")) && !this.Mflag) {
            if (this.vflag) {
                this.output(this.formatMsg("out.ignore.entry", entryName));
            }
            return;
        }
        else {
            long n = directory ? 0L : file.length();
            if (this.vflag) {
                this.out.print(this.formatMsg("out.adding", entryName));
            }
            ZipEntry e = new ZipEntry(entryName);
            e.setTime(file.lastModified());
            if (n == 0L) {
                e.setMethod(0);
                e.setSize(0L);
                e.setCrc(0L);
            }
            else if (this.flag0) {
                this.crc32File(e, file);
            }
            zipOutputStream.putNextEntry(e);
            if (directory == 0) {
                this.copy(file, zipOutputStream);
            }
            zipOutputStream.closeEntry();
            if (this.vflag) {
                long size = e.getSize();
                long compressedSize = e.getCompressedSize();
                this.out.print(this.formatMsg2("out.size", String.valueOf(size), String.valueOf(compressedSize)));
                if (e.getMethod() == 8) {
                    long l = 0L;
                    if (size != 0L) {
                        l = (size - compressedSize) * 100L / size;
                    }
                    this.output(this.formatMsg("out.deflated", String.valueOf(l)));
                }
                else {
                    this.output(this.getMsg("out.stored"));
                }
            }
            return;
        }
    }
    
    private void copy(InputStream inputStream, OutputStream outputStream) throws IOException {
        int read;
        while ((read = inputStream.read(this.copyBuf)) != -1) {
            outputStream.write(this.copyBuf, 0, read);
        }
    }
    
    private void copy(File file, OutputStream outputStream) throws IOException {
        FileInputStream fileInputStream = new FileInputStream(file);
        try {
            this.copy(fileInputStream, outputStream);
        }
        finally {
            fileInputStream.close();
        }
    }
    
    private void copy(InputStream inputStream, File file) throws IOException {
        FileOutputStream fileOutputStream = new FileOutputStream(file);
        try {
            this.copy(inputStream, fileOutputStream);
        }
        finally {
            fileOutputStream.close();
        }
    }
    
    private void crc32Manifest(ZipEntry zipEntry, Manifest manifest) throws IOException {
        Main.CRC32OutputStream out = new Main.CRC32OutputStream();
        manifest.write(out);
        out.updateEntry(zipEntry);
    }
    
    private void crc32File(ZipEntry zipEntry, File file) throws IOException {
        Main.CRC32OutputStream crc32OutputStream = new Main.CRC32OutputStream();
        this.copy(file, crc32OutputStream);
        if (crc32OutputStream.n != file.length()) {
            throw new JarException(this.formatMsg("error.incorrect.length", file.getPath()));
        }
        crc32OutputStream.updateEntry(zipEntry);
        return;
    }
    
    void replaceFSC(String[] array) {
        if (array != null) {
            for (int i = 0; i < array.length; ++i) {
                array[i] = array[i].replace(File.separatorChar, '/');
            }
        }
    }
    
    Set<ZipEntry> newDirSet() {
        return new HashSet<ZipEntry>() {
            @Override
            public boolean add(ZipEntry e) {
                return e != null && !Main.useExtractionTime && super.add(e);
            }
        };
    }
    
    void updateLastModifiedTime(Set<ZipEntry> set) throws IOException {
        for (ZipEntry zipEntry : set) {
            long time = zipEntry.getTime();
            if (time != -1L) {
                String safeName = this.safeName(zipEntry.getName().replace(File.separatorChar, '/'));
                if (safeName.length() != 0) {
                    new File(safeName.replace('/', File.separatorChar)).setLastModified(time);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
    }
    
    void extract(InputStream in, String[] array) throws IOException {
        ZipInputStream zipInputStream = new ZipInputStream(in);
        Set<ZipEntry> dirSet = this.newDirSet();
        ZipEntry nextEntry;
        while ((nextEntry = zipInputStream.getNextEntry()) != null) {
            if (array == null) {
                dirSet.add(this.extractFile(zipInputStream, nextEntry));
            }
            else {
                String name = nextEntry.getName();
                int length = array.length;
                int i = 0;
                while (i < length) {
                    if (name.startsWith(array[i])) {
                        dirSet.add(this.extractFile(zipInputStream, nextEntry));
                        break;
                    }
                    ++i;
                }
            }
        }
        this.updateLastModifiedTime(dirSet);
    }
    
    void extract(String name, String[] array) throws IOException {
        ZipFile zipFile = new ZipFile(name);
        Set<ZipEntry> dirSet = this.newDirSet();
        Enumeration<Object> entries = zipFile.entries();
        while (entries.hasMoreElements()) {
            ZipEntry zipEntry = (ZipEntry)entries.nextElement();
            if (array == null) {
                dirSet.add(this.extractFile(zipFile.getInputStream(zipEntry), zipEntry));
            }
            else {
                String name2 = zipEntry.getName();
                int length = array.length;
                int i = 0;
                while (i < length) {
                    if (name2.startsWith(array[i])) {
                        dirSet.add(this.extractFile(zipFile.getInputStream(zipEntry), zipEntry));
                        break;
                    }
                    ++i;
                }
            }
        }
        zipFile.close();
        this.updateLastModifiedTime(dirSet);
    }
    
    ZipEntry extractFile(InputStream inputStream, ZipEntry zipEntry) throws IOException {
        ZipEntry zipEntry2 = null;
        String safeName = this.safeName(zipEntry.getName().replace(File.separatorChar, '/'));
        if (safeName.length() == 0) {
            return zipEntry2;
        }
        File file = new File(safeName.replace('/', File.separatorChar));
        if (zipEntry.isDirectory()) {
            if (file.exists()) {
                if (!file.isDirectory()) {
                    throw new IOException(this.formatMsg("error.create.dir", file.getPath()));
                }
            }
            else {
                if (!file.mkdirs()) {
                    throw new IOException(this.formatMsg("error.create.dir", file.getPath()));
                }
                zipEntry2 = zipEntry;
            }
            if (this.vflag) {
                this.output(this.formatMsg("out.create", safeName));
            }
        }
        else {
            if (file.getParent() != null) {
                File file2 = new File(file.getParent());
                if ((!file2.exists() && !file2.mkdirs()) || !file2.isDirectory()) {
                    throw new IOException(this.formatMsg("error.create.dir", file2.getPath()));
                }
            }
            try {
                this.copy(inputStream, file);
            }
            finally {
                if ((inputStream instanceof ZipInputStream)) {
                    ((ZipInputStream)inputStream).closeEntry();
                }
                else {
                    inputStream.close();
                }
            }
            if (this.vflag) {
                if (zipEntry.getMethod() == 8) {
                    this.output(this.formatMsg("out.inflated", safeName));
                }
                else {
                    this.output(this.formatMsg("out.extracted", safeName));
                }
            }
        }
        if (!Main.useExtractionTime) {
            long time = zipEntry.getTime();
            if (time != -1L) {
                file.setLastModified(time);
            }
        }
        return zipEntry2;
    }
    
    void list(InputStream in, String[] array) throws IOException {
        ZipInputStream zipInputStream = new ZipInputStream(in);
        ZipEntry nextEntry;
        while ((nextEntry = zipInputStream.getNextEntry()) != null) {
            zipInputStream.closeEntry();
            this.printEntry(nextEntry, array);
        }
    }
    
    void list(String name, String[] array) throws IOException {
        ZipFile zipFile = new ZipFile(name);
        Enumeration<Object> entries = zipFile.entries();
        while (entries.hasMoreElements()) {
            this.printEntry((ZipEntry)entries.nextElement(), array);
        }
        zipFile.close();
    }
    
    void dumpIndex(String pathname, JarIndex jarIndex) throws IOException {
        File file = new File(pathname);
        Path path = file.toPath();
        Path path2 = createTempFileInSameDirectoryAs(file).toPath();
        try {
            if (this.update(Files.newInputStream(path, new OpenOption[0]), Files.newOutputStream(path2, new OpenOption[0]), null, jarIndex)) {
                try {
                    Files.move(path2, path, new CopyOption[] { StandardCopyOption.REPLACE_EXISTING });
                }
                catch (IOException cause) {
                    throw new IOException(this.getMsg("error.write.file"), cause);
                }
            }
        }
        finally {
            Files.deleteIfExists(path2);
        }
    }
    
    List<String> getJarPath(String e) throws IOException {
        ArrayList<String> list = new ArrayList();
        list.add(e);
        this.jarPaths.add(e);
        String substring = e.substring(0, Math.max(0, e.lastIndexOf(47) + 1));
        JarFile jarFile = new JarFile(e.replace('/', File.separatorChar));
        if (jarFile != null) {
            Manifest manifest = jarFile.getManifest();
            if (manifest != null) {
                Attributes mainAttributes = manifest.getMainAttributes();
                if (mainAttributes != null) {
                    String value = mainAttributes.getValue(Attributes.Name.CLASS_PATH);
                    if (value != null) {
                        StringTokenizer stringTokenizer = new StringTokenizer(value);
                        while (stringTokenizer.hasMoreTokens()) {
                            String nextToken = stringTokenizer.nextToken();
                            if (!nextToken.endsWith("/")) {
                                String concat = substring.concat(nextToken);
                                if (!this.jarPaths.contains(concat)) {
                                    list.addAll(this.getJarPath(concat));
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
        }
        jarFile.close();
        return list;
    }
    
    void genIndex(String s, String[] array) throws IOException {
        List<String> jarPath = this.getJarPath(s);
        int n = jarPath.size();
        if (n == 1 && array != null) {
            for (int i = 0; i < array.length; ++i) {
                jarPath.addAll(this.getJarPath(array[i]));
            }
            n = jarPath.size();
        }
        this.dumpIndex(s, new JarIndex((String[])jarPath.toArray(new String[n])));
    }
    
    void printEntry(ZipEntry zipEntry, String[] array) throws IOException {
        if (array == null) {
            this.printEntry(zipEntry);
        }
        else {
            String name = zipEntry.getName();
            int length = array.length;
            int i = 0;
            while (i < length) {
                if (name.startsWith(array[i])) {
                    this.printEntry(zipEntry);
                    return;
                }
                ++i;
            }
        }
    }
    
    void printEntry(ZipEntry zipEntry) throws IOException {
        if (this.vflag) {
            StringBuilder sb = new StringBuilder();
            String string = Long.toString(zipEntry.getSize());
            for (int i = 6 - string.length(); i > 0; --i) {
                sb.append(' ');
            }
            sb.append(string).append(' ').append(new Date(zipEntry.getTime()).toString());
            sb.append(' ').append(zipEntry.getName());
            this.output(sb.toString());
        }
        else {
            this.output(zipEntry.getName());
        }
    }
    
    void usageError() {
        this.error(this.getMsg("usage"));
    }
    
    void fatalError(Exception ex) {
        ex.printStackTrace();
    }
    
    void fatalError(String str) {
        this.error(this.program + ": " + str);
    }
    
    protected void output(String x) {
        this.out.println(x);
    }
    
    protected void error(String x) {
        this.err.println(x);
    }
    
    public static void main(String[] array) {
        System.exit(new Main(System.out, System.err, "jar").run(array) ? 0 : 1);
    }
    
    private File createTemporaryFile(String prefix, String s) {
        File file = null;
        try {
            file = File.createTempFile(prefix, s);
        }
        catch (IOException | SecurityException ex) {}
        if (file == null) {
            if (this.fname != null) {
                try {
                    file = File.createTempFile(this.fname, ".tmp" + s, new File(this.fname).getAbsoluteFile().getParentFile());
                }
                catch (IOException ex2) {
                    this.fatalError(ex2);
                }
            }
            else {
                this.fatalError(new IOException(this.getMsg("error.create.tempfile")));
            }
        }
        return file;
    }
    
    static {
        useExtractionTime = Boolean.getBoolean("sun.tools.jar.useExtractionTime");
        try {
            Main.rsrc = ResourceBundle.getBundle("sun.tools.jar.resources.jar");
        }
        catch (MissingResourceException ex) {
            throw new Error("Fatal: Resource for jar is missing");
        }
    }
    
    private static class CRC32OutputStream extends OutputStream
    {
        final CRC32 crc;
        long n;
        
        CRC32OutputStream() {
            this.crc = new CRC32();
            this.n = 0L;
        }
        
        @Override
        public void write(int b) throws IOException {
            this.crc.update(b);
            ++this.n;
        }
        
        @Override
        public void write(byte[] b, int off, int len) throws IOException {
            this.crc.update(b, off, len);
            this.n += (long)len;
        }
        
        public void updateEntry(ZipEntry zipEntry) {
            zipEntry.setMethod(0);
            zipEntry.setSize(this.n);
            zipEntry.setCrc(this.crc.getValue());
        }
    }
}
